<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\User;
use App\Mail\SendOtpMail;
use Illuminate\Http\Request;
use PhpParser\Node\Stmt\TryCatch;
use App\Repositories\UserRepository;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    protected $userRepository;

    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    public function signup(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'first_name' => 'required|string|max:255',
            'last_name' => 'required|string|max:255',
            'email' => 'required|email|unique:users,email',
            'phone' => 'required|string|max:15|unique:users,phone',
            'password' => 'required|min:8'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => 'Validation errors',
                'errors' => $validator->errors(),
            ], 422);
        }
        $data = $request->only('first_name', 'last_name', 'email', 'phone', 'password');
        $data['password'] = Hash::make($request->password);
        $user = $this->userRepository->create($data);
        $userResponse = (object) [
            'success'   =>  true,
            'message'   =>  'User Created Successfully',
            'data'      =>  $user
        ];
        return response()->json($userResponse, 201);
    }


    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|string',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'error',
                'message' => 'Validation errors',
                'errors' => $validator->errors(),
            ], 422);
        }

        // Attempt to log the user in using the UserRepository
        $user = $this->userRepository->attemptLogin($request->only('email', 'password'));

        // If login fails, return error
        if (!$user) {
            return response()->json([
                'status' => 'error',
                'message' => 'Invalid credentials',
            ], 401);
        }

        return response()->json([
            'status' => 'success',
            'message' => 'Login successful',
            'data' => [
                'user' => $user,
            ],
        ], 200);
    }

    public function getUserProfile()
    {
        $user = User::find(auth('sanctum')->user()->id);
        return response()->json(['success' => true, 'message' => 'user profile', 'data' => $user]);
    }

    public function updateProfile(Request $request)
    {
        $user = User::find(auth('sanctum')->user()->id);

        if (isset($request->first_name) && !is_null($request->first_name)) {
            $user->first_name = $request->first_name;
        }

        if (isset($request->last_name) && !is_null($request->last_name)) {
            $user->last_name = $request->last_name;
        }

        if (isset($request->email) && !is_null($request->email)) {
            $user->email = $request->email;
        }

        if (isset($request->phone) && !is_null($request->phone)) {
            $user->phone = $request->phone;
        }

        if (isset($request->birthday) && !is_null($request->birthday)) {
            $user->birthday = $request->birthday;
        }

        if (isset($request->gender) && !is_null($request->gender)) {
            $user->gender = $request->gender;
        }

        if (isset($request->nationality) && !is_null($request->nationality)) {
            $user->nationality = $request->nationality;
        }


        if ($user->isDirty()) {
            $user->save();
            return response()->json([
                'success' => 'true',
                'message' => 'User updated successfully',
                'data' => $user
            ]);
        } else {
            return response()->json([
                'success' => 'false',
                'message' => 'No changes detected',
                'data' => []
            ]);
        }
    }


    public function changePassword(Request $request)
    {
        // Validate the incoming request
        $validator = Validator::make($request->all(), [
            'current_password' => 'required',
            'new_password' => 'required|min:8', // The 'confirmed' rule checks if there's a 'new_password_confirmation' field matching the 'new_password'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'errors' => $validator->errors(),
            ], 400);
        }

        // Get the currently authenticated user
        $user = $request->user();


        // Check if the current password matches
        if (!Hash::check($request->current_password, $user->password)) {
            return response()->json([
                'success' => false,
                'message' => 'Current password does not match',
            ], 400);
        }

        // Update the user's password
        $user->password = Hash::make($request->new_password);
        $user->save();

        return response()->json([
            'success' => true,
            'message' => 'Password changed successfully',
            'data' => []
        ]);
    }


    public function forgotPassword(Request $request)
    {
        $request->validate(['email' => 'required|email']);

        try {
            $user = User::where('email', $request->email)->first();


            if (!$user) {
                return response()->json(['success' => false, 'message' => 'User not found', 'data' => []], 404);
            }

            $otp = rand(100000, 999999);

            // Set OTP and expiration time (e.g., 10 minutes from now)
            $user->otp = $otp;
            $user->otp_expires_at = Carbon::now()->addMinutes(10);
            $user->save();

            Mail::to($user->email)->send(new SendOtpMail($otp));

            return response()->json(['success' => true, 'message' => 'OTP has sent to your  email', 'data' => []]);
        } catch (\Throwable $th) {
            return response()->json(['success' => false, 'message' => $th->getMessage(), 'data' => []]);
        }
    }


    public function verifyOtpPassword(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'otp'   => 'required|string'
        ]);

        $user = User::where('email', $request->email)->first();


        if (!$user || $user->otp != $request->otp) {
            return response()->json(['success' => false, 'message' => 'Invalid OTP', 'data' => []], 400);
        }

        if (Carbon::now()->isAfter($user->otp_expires_at)) {
            return response()->json(['success' => false, 'message' => 'OTP expired', 'data' => []], 400);
        }

        return response()->json([
            'success' => true,
            'message' => 'OTP verified',
            'data' => $user
        ]);
    }



    public function updatePassword(Request $request)
    {
        $request->validate([
            'email'    => 'required|email',
            'password' => 'required|string|min:8|confirmed'
        ]);

        $user = User::where('email', $request->email)->first();

        // Update password and clear OTP
        $user->password = Hash::make($request->new_password);
        $user->otp = null;
        $user->otp_expires_at = null;
        $user->save();

        return response()->json([
            'success' => true,
            'message' => 'Password updated successfully',
            'data' => []
        ]);
    }
}
